#include "myframelistener.h"
#include <Ogre.h>
#include <OgreErrorDialog.h>

using namespace Ogre;
using namespace std;

int main(int, char **)
{
  try
  {
    //////////////////////////////////////////////////////////////
    // Initialization of Root and RenderWindow
    //////////////////////////////////////////////////////////////
    Root* mRoot = new Root("../../plugins.cfg","../ogre.cfg","ogre.log");
    if (!mRoot->restoreConfig())
    {
      if (!mRoot->showConfigDialog())
      {
        return 1;
      }
    }
    RenderWindow* mWindow = mRoot->initialise(true,"Application");
    //////////////////////////////////////////////////////////////
    // Initializing resources
    //////////////////////////////////////////////////////////////
    // Load resource paths from config file
    ConfigFile cf;
    cf.load("../../resources.cfg");    
    // Go through all sections & settings in the file
    ConfigFile::SectionIterator seci = cf.getSectionIterator();    
    while (seci.hasMoreElements())
    {
      String secName = seci.peekNextKey();
      ConfigFile::SettingsMultiMap* settings = seci.getNext();
      for (ConfigFile::SettingsMultiMap::iterator i = settings->begin();
           i != settings->end(); ++i)
      {
        String typeName = i->first;
        String archName = i->second;
        ResourceGroupManager::getSingleton().addResourceLocation(
          archName, typeName, secName);
      }
    }
    ResourceGroupManager::getSingleton().initialiseAllResourceGroups();
    //////////////////////////////////////////////////////////////
    // Initialization of Scene
    //////////////////////////////////////////////////////////////
    SceneManager* mSceneManager = mRoot->createSceneManager(ST_GENERIC,"sceneManager");
    Camera* mCamera = mSceneManager->createCamera("Camera");
    mCamera->setPosition(Vector3(0.0f,0.0f,500.0f));
    mCamera->lookAt(Vector3(0.0f,0.0f,0.0f));
    mCamera->setNearClipDistance(5.0f);
    mCamera->setFarClipDistance(5000.0f);

    mSceneManager->setAmbientLight( ColourValue( 1, 1, 1 ) );
      
    Entity *ent1 = mSceneManager->createEntity( "Robot1", "robot.mesh" );
    SceneNode* node1 =
      mSceneManager->getRootSceneNode()->createChildSceneNode("Robot1Node");
    node1->attachObject(ent1);
    ////////////////////////////////////////////////////////////
    // Binding of Scene and RenderWindow
    ////////////////////////////////////////////////////////////
    Viewport* mViewport = mWindow->addViewport(mCamera);
    mViewport->setBackgroundColour(ColourValue(0.0f,0.0f,0.0f));
    mCamera->setAspectRatio(Real(mViewport->getActualWidth()) /
                            Real(mViewport->getActualHeight()));
    ////////////////////////////////////////////////////////////
    // Managing input
    ////////////////////////////////////////////////////////////
    InputReader* mInputReader = PlatformManager::getSingleton().createInputReader();
    mInputReader->initialise(mWindow,true,false);

    MyFrameListener* mFrameLst = new MyFrameListener(mInputReader);
    mRoot->addFrameListener(mFrameLst);
    //////////////////////////////////////////////////////////////
    // Running the main loop
    //////////////////////////////////////////////////////////////
    mRoot->startRendering();

    delete mFrameLst;
    delete mRoot;
  }
  catch (Exception& e)
  {
    ErrorDialog* dlg = PlatformManager::getSingleton().createErrorDialog();
    dlg->display(e.getFullDescription());
    delete dlg;
    return 1;
  }
  return 0;
}

